// Unit:       LaunchM
// Version:    1.00
// Purpose:    implement context menu to Launch KEdit32 for specified file
//
// Author:     Peter Sawatzki (ps)
//             Buchenhof 3, 58091 Hagen, Germany
// CompuServe: 100031,3002
// E-Mail:     Peter@Sawatzki.de
// web:        http://www.sawatzki.de
//
// Date:    Author:
// 07/21/97 ps    first version
// 02/05/98 ps    fixed () bug
//
// Copyright  1996,1997 Peter Sawatzki. All Rights Reserved.

unit LaunchM;
interface
uses
    Windows, ComObj, ComServ, ShlObj, ActiveX, ShellApi, SysUtils, IniFiles;

Const
   CLSID_ContextMenuShellExtension: TGUID = '{8D1B3EF0-1C73-11D1-99B1-0000929B06C1}';
type
    TContextMenu = class(TComObject, IShellExtInit, IContextMenu)
    private
      szFile: array[0..MAX_PATH] of Char;
    public
      function QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst, idCmdLast,
        uFlags: UINT): HResult; stdcall;
      function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
      function GetCommandString(idCmd, uFlags: UINT; pwReserved: PUINT;
        pszName: LPSTR; cchMax: UINT): HResult; stdcall;
      function Initialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
        hKeyProgID: HKEY): HResult; stdcall;
    end;

Implementation
Const
  KEditModuleDefault = 'KEditW32.Exe';
  KEditIni = 'KEditW.Ini';
Var
  KEditModule: String = '';

Function GetKEditPath: string;
// Returns string containing path KEdit
begin
  If KEditModule='' Then Begin
    KEditModule:= KEditModuleDefault;
    With TIniFile.Create(KEditIni) Do
    Try
      KEditModule:= ReadString('State32', 'ModuleName', KEditModuleDefault)
    Finally
      Free
    End
  End;
  Result:= KEditModule+' "%s"'
end;

function TContextMenu.QueryContextMenu(Menu: HMENU; indexMenu, idCmdFirst,
          idCmdLast, uFlags: UINT): HResult;
begin
  // Add one menu item to context menu
  InsertMenu (Menu, indexMenu, MF_STRING or MF_BYPOSITION, idCmdFirst,
    '&KEdit');
  // Return number of menu items added
  Result:= 1
end;

Function TContextMenu.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
Var
  SI: TStartupInfo;
  PI: TProcessInformation;
  TheDir: String;
Begin
  // Make sure we are not being called by an application
  If HiWord(Integer(lpici.lpVerb)) <> 0 then begin
    Result := E_FAIL;
    Exit
  End;
  // Make sure we aren't being passed an invalid argument number
  If LoWord(lpici.lpVerb) > 0 then begin
    Result := E_INVALIDARG;
    Exit
  End;
  // Execute the command specified by lpici.lpVerb.
  If LoWord(lpici.lpVerb)=0 Then Begin
    // invoke KEdit
    FillChar(SI, SizeOf(SI), 0);
    FillChar(PI, SizeOf(PI), 0);
    SI.cb:= SizeOf(SI);
    TheDir:= ExtractFileDir(szFile);
    CreateProcess(Nil, PChar(Format(GetKEditPath, [szFile])), Nil, Nil,
      False, 0, Nil, PChar(TheDir), SI, PI);
    CloseHandle(PI.hThread);
    CloseHandle(PI.hProcess);
  End;
  Result := NOERROR
End;

function TContextMenu.GetCommandString(idCmd, uFlags: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HRESULT;
Const
  HelpText = 'Edit file with KEdit32';
begin
  if idCmd = 0 then begin
    // return help string for menu item
    If uFlags=GCS_HELPTEXTW Then
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, HelpText, -1, PWideChar(pszName), cchMax)
    Else
      StrCopy(pszName, HelpText);
    Result := NOERROR;
  end else
    Result := E_INVALIDARG;
end;

function TContextMenu.Initialize(pidlFolder: PItemIDList; lpdobj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  medium: TStgMedium;
  fe: TFormatEtc;
begin
  with fe do
  begin
    cfFormat := CF_HDROP;
    ptd := Nil;
    dwAspect := DVASPECT_CONTENT;
    lindex := -1;
    tymed := TYMED_HGLOBAL;
  end;
  // Fail the call if lpdobj is Nil.
  if lpdobj = Nil then begin
    Result := E_FAIL;
    Exit;
  end;
  // Render the data referenced by the IDataObject pointer to an HGLOBAL
  // storage medium in CF_HDROP format.
  Result := lpdobj.GetData(fe, medium);
  if Failed(Result) then Exit;
  // If only one file is selected, retrieve the file name and store it in
  // szFile. Otherwise fail the call.
  if DragQueryFile(medium.hGlobal, $FFFFFFFF, Nil, 0) = 1 then begin
    DragQueryFile(medium.hGlobal, 0, szFile, SizeOf(szFile));
    Result := NOERROR;
  end else
    Result := E_FAIL;
  ReleaseStgMedium(medium)
end;

initialization
  TComObjectFactory.Create(ComServer, TContextMenu, CLSID_ContextMenuShellExtension,
       '', 'KEdit launcher', ciMultiInstance);
end.
